#!/bin/bash

######################################################################################################################################
#
# This is a simple shell script to help anyone kickstart the nfs-ganesha server for use with the GlusterFS filesystem/ Red Hat Storage
# Server.
#
# The script is a basic one and has several limitations. Please follow the Release Notes and Guide completely before using it.
#
# The script has some pre-requisites. It assumes:
#	1. the availability of some test GlusterFS volume which will be exported via nfs-ganesha
#	2. IPv6 is enabled for the nfs-ganesha server IP, even if you don't use V6. Please see the Release Notes for details.
#	   Else nfs-ganesha daemon may not run.
#
# The script does the following:
#	- it will help configure the nfs-ganesha conf file for exporting a single GlusterFS volume
#	- it will automatically start the nfs-ganesha server once the nfs-ganesha conf file is populated with the supplied parameters
#	- it will disable kernel NFS services since kernel NFS and nfs-ganesha cannot co-exist
#		*** This means kernel nfs services will be disabled for the host server *** If you are using kernel nfs or other nfs
#		*** services on the host for other purposes or applications, this means those could be impacted ***
#	- it will disable NFS for the GlusterFS volume which is being exported via nfs-ganesha. Both cannot co-exist!
#	  *** It is important you do not try to export the same GlusterFS volume via Gluster-NFS and nfs-ganesha simultaneously ***
#	- it will stop nfs-ganesha based on the command line option but it does not restore gluster-NFS support for the GlusterFS
#	  volume in question
#
######################################################################################################################################
VOLNAME=
CONF=
IP=
LOG=
DBGLVL=
OPT=
FILE1=
DBG=
trap    cleanup   SIGHUP SIGINT SIGTERM


        if [ $EUID -ne 0 ];  then
                echo "You need root privileges to run this script."
                exit 1
        fi


function usage 
{
        echo " Usage : rhs-nfs_ganesha.sh [-l <logfile>][-n <log_level>] -v <volname>
		                          -i <ganesha_host_ip> -f <config_file>
                                          -c <start|stop>

               [-h]                    usage, display help.

               [-l <logfile>]          logfile for nfs-ganesha server logs.

               [-n <log_level>]        set the verbosity level.
	       log level options       value in [0-6], 0 being the lowest level of
                                       verbosity and the 6 the highest.
                                       The values are:
				       0=NONE, 1=MAJOR ONLY, 2=CRITICAL,
                                       3=EVENTS, 4=DEBUG, 5=MORE DEBUG,
                                       6=FULL DEBUG

               -v <volname>            name of the volume to be exported.

               -i <ganesha_host_ip>    IP of the ganesha host or server.

               -f <config_file>        the nfs-ganesha config file to be used.

               -c <start|stop>         command to start or stop the nfs-ganesha
                                       server.

                =================== Default Values ====================
                LogFile    	        /tmp/nfs-ganesha.log

                DebugLevel 		3 (EVENTS)"
}

while getopts "hf:i:l:n:v:c: -l help" OPTION
           do
             case $OPTION in
                     f) CONF=$OPTARG
                     ;;
                     i) IP=$OPTARG
                     ;;
                     l) LOG=$OPTARG
                     ;;
                     n) DBGLVL=$OPTARG
                     ;;
                     h) usage 
			exit 1
                     ;;
                     v) VOLNAME=$OPTARG
                     ;;
                     c) OPT=$OPTARG
                     ;;
             esac
     done


function cleanup 
{
	if [ -f /tmp/old-ganesha.conf ] 
	then  cp /tmp/old-ganesha.conf $CONF 
	exit 1
	fi 
}


function check_for_stop
{
	if  echo $OPT | grep -i -q  "stop" 
	then stop_ganesha
	fi
}

function check_usage
{

     if [ "$VOLNAME" = "" ]
     then
             usage
             exit 1
     fi
     if [ "$CONF" = "" ]
     then
             usage
             exit 1
     fi
     if [ "$IP" = "" ]
     then
             usage
     exit 1
     fi
     if [ "$OPT" = "" ]
     then
             usage
     exit 1
     fi
}

function check_ip
{
        if [[ ! $IP =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then
                echo "Invalid  IP, please enter the correct IP of the ganesha host."
                usage
                exit 1
        fi
	if ! ifconfig -a | grep -q $IP
	then
		echo "IP address of the ganesha host doesn't seem valid, please check, exiting."
		exit 1
        fi
}


function check_volname
{
        if ! ls /var/lib/glusterd/vols | grep -q "$VOLNAME" ;  then
                echo "Volume doesn't exist. Please enter a valid volume name."
                exit 1
        fi
}

function check_conf
{

        if ! [ -f "$CONF" ]; then
                echo "The config_file $CONF doesn't exist."
                usage
                exit 1
        else
                if ! [ -s "$CONF" ]; then
                        echo "The config_file $CONF is empty. "
                        exit 1
                fi
        fi
}

function  check_option
{
        if ! echo $OPT | grep -i  -q  -e "start"  -q -e "stop"
        then
                        echo  "Invalid value for option 'c' specified : start or stop expected."
                                exit 1
        else if echo $OPT | grep -i -q "start"
        then
                OPT="start"
        else
                OPT="stop"
        	fi
        fi
}

function check_debug
{
        if [ "$DBGLVL" = "" ] ; then
                DBG="NIV_EVENT"
        else
                if ! [[ $DBGLVL -ge 0 && $DBGLVL -lt 7 ]]
                then
                        echo  "Invalid value for option 'n': an integer between [0-6] is expected."
			usage
                        exit 1
                fi
        fi

	case $DBGLVL in
                     0) DBG="NIV_NULL"
                     ;;
                     1)  DBG="NIV_MAJ"
                     ;;
                     2)  DBG="NIV_CRIT"
                     ;;
                     3)  DBG="NIV_EVENT"
                     ;;
                     4)  DBG="NIV_DEBUG"
                     ;;
                     5)  DBG="NIV_MID_DEBUG"
                     ;;
                     6)  DBG="NIV_FULL_DEBUG"
                     ;;
          esac
}

function check_logfile
{
        if [ "$LOG" = "" ]; then
                LOG="/tmp/nfs-ganesha.log"
        fi
}

function call_checks
{
        check_ip
        check_conf
	cp $CONF /tmp/old-ganesha.conf

        check_debug
        check_logfile
        check_option
        check_volname
}

function check_glusterd
{
        if ! ps aux | grep -q -e "[g]lusterd$" -q -e "[g]lusterd.pid"
        then
                echo "glusterd not started, please start glusterd. "
                exit 1
        fi
}

function check_glusternfs
{
        if  echo "$(ls /var/lib/glusterd/vols | wc -l)" | grep -q "1" ; then
                if   ps aux | grep -q "[g]luster/nfs" ; then
                echo "gluster-nfs server is active, needs to be disabled to proceed. "
                while true; do
                        read -p "Do you wish to disable gluster-nfs server ? [Y\N]" yn
                        case $yn in
                                [Yy]* ) gluster volume set $VOLNAME nfs.disable ON >/dev/null 2>/dev/null; break;;
                                [Nn]* ) exit 1;;
                                * ) echo "Please answer yes or no.";;
                        esac
                done
                sleep 2
		if ! less /var/lib/glusterd/vols/$VOLNAME/info | grep -q "nfs.disable=ON"
		then
                echo "gluster-nfs server is active, needs to be disabled to proceed. "
                while true; do
                        read -p "Do you wish to disable gluster-nfs server ? [Y\N]" yn
                        case $yn in
                                [Yy]* ) gluster volume set $VOLNAME nfs.disable ON >/dev/null 2>/dev/null; break;;
                                [Nn]* ) exit 1;;
                                * ) echo "Please answer yes or no.";;
                        esac
                done
                sleep 2
		fi
                if ps aux | grep -q "[g]luster/nfs" ; then
                                 echo "Volume set unsuccessful, please try disabling gluster-nfs server."
                                exit 1
                         fi
                sleep 2
                        if rpcinfo -p | grep -q -e  "nfs_acl" ; then
                                 echo "NFS ports still appear to be in use, please try again."
                        exit 1
                        fi

         fi
 else
         if    ps aux | grep -q  "[g]luster/nfs"; then
                echo "gluster-nfs server is active, needs to be disabled to proceed."
                exit 1
                 fi
        fi
}

function check_kernelnfs
{
        service nfs status  | grep -q -e "running" -q -e "active"  
        if [ "$?" -eq 0 ]; then
                echo "kernel-NFS server is active, it needs to disabled to proceed. "
                while true; do
                        read -p "Do you wish to disable kernel-nfs server ?  [Y\N]  " yn
                                case $yn in
                                         [Yy]* ) service nfs stop; break;;
                                         [Nn]* ) exit 1;;
                                          * ) echo "Please answer yes or no.";;
                                esac
                done
                sleep 2
                service nfs status | grep -q -e "dead" -q -e "stopped" -q -e "inactive" >>/dev/null
                if [ "$?" -eq 1 ]; then
                        echo "kernel-NFS server couldn't be disabled, please try again."
                        exit 1
                else
                        echo "kernel-NFS server successfully disabled."
                fi
        fi
}


function check_volstart
{

        if ! cat /var/lib/glusterd/vols/$1/info | grep -q "status=1" 
        then
                echo "Volume $1 is not started, please start the volume."
                exit 1
        fi
}

function check_ganesha
{
        if  ps aux | grep -q "[g]anesha.nfsd" ;  then
                echo "NFS-ganesha server is already active, nothing to do."
                exit 1
        fi
}


function check_ports
{
        if netstat -an | grep -q "2049$" ; then
                echo "Port 2049 is already in use, unable to proceed."
                exit 1
        fi
}

function check_libgfapi
{

if !  ls /usr/lib64 | grep -q "libgfapi.so"
         then


               echo "Cannot find shared object libgfapi.so in /usr/lib64/, unable to proceed."
               exit 1
       fi
}

function check_fsal
{

       if !  ls /usr/lib64/ganesha | grep -q "libfsalgluster.so"  
	 then
		
		
               echo "Cannot find shared object libfsalgluster.so, unable to proceed."
               exit 1
       fi
	if ! ls /usr/lib64/ganesha | grep -q "libfsalgluster.so$"
        then
        ln -s /usr/lib64/ganesha/libfsalgluster.so.4 /usr/lib64/ganesha/libfsalgluster.so
        fi
       


}

function check_rpms
{
	if !  rpm -qa | grep -q  nfs-ganesha-2.0.0.*.el6rhs.x86_64
	then
	echo "nfs-ganesha-2.0.0.*.el6rhs.x86_64 rpms need to be installed, exiting."
	exit 1
	fi
}
 
function start_ganesha
{
	check_ganesha
        check_rpms
	check_libgfapi
        check_fsal
        check_glusterd
        check_volstart $1
        check_kernelnfs
        check_glusternfs  $1
        check_ports
        check_conf $3
        sed -i /^[[:space:]]*\#/!s/volume.*/"volume=$1,hostname=$2\";"/ $3
        sed -i /^[[:space:]]*\#/!s/Pseudo.*/Pseudo="\"\/$1\";"/ $3
        sed -i s/Path.*/Path="\"\/$1\";"/ $3
        sed -i 's/\r//g' $3
	if ls /usr/bin | grep -q "ganesha.nfsd" 
	then
        /usr/bin/ganesha.nfsd -f $CONF  -L  $LOG -N $DBG -d  >/dev/null 2>/dev/null
	fi
	sleep 1

}

function check_rpcinfo
{
	if  rpcinfo -p | grep -q "nfs"
	then
		echo "NFS-ganesha server could not to be stopped cleanly. Please try again."
		exit 1
	else
		echo "NFS-ganesha server is stopped."
		exit 0
	fi
}
	
function clean_rpcinfo
{
                rpcinfo -d 100003 3
                rpcinfo -d 100003 4
                rpcinfo -d 100003 1
                rpcinfo -d 100005 1
                rpcinfo -d 100005 3
                rpcinfo -d 100021 4
                rpcinfo -d 100011 1
                rpcinfo -d 100011 2
                gluster volume start $VOLNAME force >/dev/null 2>/dev/null
}

function stop_ganesha2
{
	if ! ps aux | grep -q "[g]anesha.nfsd" ;    #Check if NFS-ganesha was killed by kill -s TERM.
	then
		if rpcinfo -p | grep -q "nfs"       #Check if ports were unregistered cleanly.
		then
			clean_rpcinfo
			check_rpcinfo
		else
			echo "NFS-ganesha server is stopped."
			exit 0
		fi
	else                                       #If NFS-ganesha wasn't killed by kill -s TERM
		kill -9 `cat /tmp/ganesha.pid`
		sleep 2
		rm -rf /tmp/ganesha.pid
		if   ps aux | grep -q "[g]anesha.nfsd";
		then
			echo "NFS-ganesha server could not be stopped, please try again."
			exit 1
		else
			clean_rpcinfo
			check_rpcinfo
		fi
        fi
	
}

function stop_ganesha
{
        if ! ps aux | grep -q "[g]anesha.nfsd" ;  then
                echo "NFS-ganesha server already stopped, nothing to be done."
                exit 1
        fi
	cp /var/run/ganesha.pid /tmp/ganesha.pid
        kill -s TERM `cat /var/run/ganesha.pid`
	sleep 5
	stop_ganesha2
}

function check_for_stop()
{
        if  echo $OPT | grep -i -q  "stop"
        then stop_ganesha
        fi
}



check_for_stop
check_usage
call_checks

if [ "$OPT" = "start"  ]
then
        start_ganesha $VOLNAME $IP $CONF
        if  ! ps aux | grep -q "[g]anesha.nfsd" ; then
                start_ganesha $VOLNAME $IP $CONF
        fi
        sleep 5
        if ! ps aux | grep  -q "[g]anesha.nfsd"
        then
                cp /tmp/old-ganesha.conf $CONF
		rm -rf /tmp/ganesha.conf
                echo "Failed to start NFS-ganesha server, please see $LOG for details."
         else
                echo ""
                echo "NFS-ganesha server started."
                echo "============Volume exports============"
                showmount -e $IP
		echo "======================================"
		rm -rf /tmp/old-ganesha.conf
         fi
fi

if [ "$OPT" = "stop" ]
then
	stop_ganesha
	exit 0
fi

